# -*- coding: UTF-8 -*-
require 'date'

#
#= 暦日算出用ロジックを付加したDateTimeクラスライブラリ
#
#Authors::   Yoshihiko Hara(GoroNeko)
#Date::      2010/08/14
#Version::   0.9.0
#Copyright:: Copyright (C) Yoshihiko Hara(GoroNeko), 2010. All rights reserved.
#License::   GNU GENERAL PUBLIC LICENSE Version 2
#

class AlmanacDateTime < DateTime

  # 本クラスライブラリの、名前とバージョン
  NAME = "AlmanacDateTime"
  VERSION = "0.9.0"

  #
  # 時間の単位を変換するための定数群
  #

  # 1日と等しい時間
  DAY_TO_HOUR = 24.0
  # 1時間と等しい分
  HOUR_TO_MINUTE = 60.0
  # 1分と等しい秒
  MINUTE_TO_SECOND = 60.0
  # 1ヶ月あたりの平均日数
  AVERAGE_MONTH_DAYS = 30.6001
  # 1年あたりの平均日数
  AVERAGE_YEAR_DAYS = 365.25

  # 世界最初の（導入国：イタリア）グレゴリオ暦導入年月日のJD
  NEW_STYLE_START_JD = 2299161.0

  # 曜日の漢字和名
  ABBR_DAYNAMES_J \
    = Array.new(["日", "月", "火", "水", "木", "金", "土"])

  # 曜日の英名
  # DAYNAMES または ABBR_DAYNAMES 参照

  # 紀元前m年を、仮想的な紀元n年に変換する。
  #
  # 【引数】
  #   1.紀元に変換したい、紀元前の年(bc:Integer)
  # 【戻り値】
  #   1.与えられた紀元前の年を変換した、仮想的な紀元(answer_ad:Integer)
  # 【備考】
  #   紀元(Anno Domini A.D.)は1年から
  #   紀元(Anno Domini A.D.)0年 = 紀元前(Before Christ B.C.)1年
  # 　∴ 紀元n年 = -( 紀元前m年 - 1 )
  #
  def bc_to_ad(bc)
    answer_ad = -( bc - 1 )
    return answer_ad
  end

  # ユリウス日(Julian day)を、紀元n年に変換する
  #
  # 【引数】
  #   1.紀元に変換したい、ユリウス日(ajd:Rational)
  #   2.時差(of:Rational)
  # 【戻り値】
  #   1.変換後の情報を格納したAlmanacDateTimeオブジェクト
  #     (answer_almanac_date_time:AlmanacDateTime)
  # 【備考】
  #   以下のプログラムは、Jean Meeus 氏の式を参考に作成した。
  #
  def ajd_to_ad(ajd, of)

    float_ajd = ajd.to_f
    float_of = of.to_f
    float_jd = self.jd.to_f

    # ユリウス日の時分秒の値（小数点以下の部分）から、
    # 時（int_hour）・分（int_minute）･秒（int_second）を求める。
    float_hour_minute_second = float_ajd + 0.5 + float_of
    float_hour_minute_second = float_hour_minute_second \
                               - float_hour_minute_second.to_i.to_f

    # 時（int_hour）算出
    float_hour_minute_second = float_hour_minute_second * DAY_TO_HOUR
    int_hour = float_hour_minute_second.to_i

    # 分（int_minute）算出
    float_hour_minute_second = float_hour_minute_second - int_hour.to_f
    float_hour_minute_second = float_hour_minute_second * HOUR_TO_MINUTE
    int_minute = float_hour_minute_second.to_i

    # 秒（int_second）算出
    float_hour_minute_second = float_hour_minute_second - int_minute.to_f
    float_hour_minute_second = float_hour_minute_second * MINUTE_TO_SECOND
    int_second = float_hour_minute_second.to_i

    # ユリウス日の年月日の値（小数点以下除くの部分）から、
    # 年（int_year）・月（int_month）･日（int_day）を求める。
    float_hour_minute_second = float_ajd + 0.5 + float_of
    float_year_month_day = float_hour_minute_second.to_i.to_f
    float_hour_minute_second = float_hour_minute_second - float_year_month_day

    # 変換対象が、ユリウス暦に存在する日付か
    # グレゴリオ暦(New Style)以降に存在する日付かにより、補正を行う
    float_Correction_year_month_day = float_year_month_day
    if float_year_month_day >= NEW_STYLE_START_JD
      float_Correction_work \
        = ((float_year_month_day - 1867216.25) / 36524.3).to_i.to_f
      float_Correction_year_month_day = float_year_month_day + 1.0 \
                                        + float_Correction_work \
                                        - (float_Correction_work / 4).to_i.to_f
    end

    float_Correction_year_month_day \
      = float_Correction_year_month_day + 1524.0
    float_Correction_year \
      = ((float_Correction_year_month_day - 122.1) / AVERAGE_YEAR_DAYS).to_i.to_f

    float_Correction_day_work = (AVERAGE_YEAR_DAYS * float_Correction_year).to_i.to_f

    float_month_work = 0.0
    float_month_work_sub_1 \
      = (float_Correction_year_month_day - float_Correction_day_work) / AVERAGE_MONTH_DAYS
    float_month_work_sub_2 = float_month_work_sub_1.to_i.to_f
    float_month_work_sub_3 = 0.0

    float_month_work = float_month_work_sub_2
    if float_month_work_sub_1 <= 0.0
      float_month_work_sub_3 \
        = (float_month_work_sub_1 - float_month_work_sub_2) * (-1)
      if float_month_work_sub_3 >= 0.5
        float_month_work -= 1.0
      end
    end

    # 日（int_day）算出
    int_day = float_Correction_year_month_day.to_i \
              - float_Correction_day_work.to_i \
              - (AVERAGE_MONTH_DAYS * float_month_work).to_i \
              + float_hour_minute_second.to_i

    # 月（int_month）算出
    if float_month_work < 13.5
      float_month = float_month_work - 1.0
    else
      float_month = float_month_work - 13.0
    end
    int_month = float_month.to_i

    # 年（int_year）算出
    if float_month > 2.5
      int_year = float_Correction_year.to_i - 4716
    else
      int_year = float_Correction_year.to_i - 4715;
    end

    # 閏年補正
    if int_month == 2 && int_day == 29
      if int_year % 100 == 0 && int_year % 400 != 0
        int_month = 3;
        int_day = 1;
      end
    end

    answer_almanac_date_time \
      = AlmanacDateTime.civil(int_year, \
                              int_month, \
                              int_day, \
                              int_hour, \
                              int_minute, \
                              int_second, of)

    return answer_almanac_date_time

  end

  # DateTimeオブジェクトを、日本における時刻（offset値：日本）に変換する
  #
  # 【引数】
  #   1.日本における時刻に変換したいAlmanacDateTimeオブジェクト
  #     (target_date_time:AlmanacDateTime)
  # 【戻り値】
  #   1.変換後の情報を格納したAlmanacDateTimeオブジェクト
  #     (answer_almanac_date_time:AlmanacDateTime)
  # 【備考】
  #   干支、九星等は、「日本における時刻」で計算する必要が多々発生するため
  #   共通ロジックとして用意した
  #
  def change_offset_to_japan(target_date_time)
    # オフセット値が日本（Rational(3, 8)）でない場合のみ
    # 以下の補正計算を実施
    if target_date_time.offset != Rational(3, 8)
      target_date_time \
        = target_date_time \
          + (Rational(3, 8) - target_date_time.offset)
    else 
      return target_date_time
    end

    answer_almanac_date_time \
      =AlmanacDateTime.civil(target_date_time.year, \
                             target_date_time.month, \
                             target_date_time.day, \
                             target_date_time.hour, \
                             target_date_time.minute, \
                             target_date_time.second, \
                             Rational(3, 8))

    return answer_almanac_date_time

  end

end
