# -*- coding: UTF-8 -*-

#
#= 度(degree)とラジアン(radian)の変換を意識した暦日計算補助用クラスライブラリ
#
#Authors::   Yoshihiko Hara(GoroNeko)
#Date::      2010/08/14
#Version::   0.9.0
#Copyright:: Copyright (C) Yoshihiko Hara(GoroNeko), 2010. All rights reserved.
#License::   GNU GENERAL PUBLIC LICENSE Version 2
#

class AlmanacMath

  # 本クラスライブラリの、名前とバージョン
  NAME = "AlmanacMath"
  VERSION = "0.9.0"

  # 度(degree)をラジアン(radian)に変換するための定数
  COEFFICIENT_OF_RADIAN_TO_DEGREE = 57.2957795

  # 1°(degree)と等しいラジアン(radian)
  RADIAN_EQUAL_TO_ONE_DEGREE = 0.017453292519943
  # 度(degree)の最大値
  MAX_OF_DEGREE = 360.0
  # 度(degree)の最小値
  MIN_OF_DEGREE = 0.0

  # ラジアン(radian)を度(degree)に変換する。
  #
  # 【引数】
  #   1.ラジアン(radian:Float)
  # 【戻り値】
  #   1.度(degree:Float)
  # 【備考】
  #   戻り値は少数下4桁まででまるめたもの
  #   誤差有り
  #
  def radian_to_degree(radian)
    degree = radian * COEFFICIENT_OF_RADIAN_TO_DEGREE
    # 計算結果が度の上限下限（0 <= n <360）を越えていた場合は補正する
    degree = put_between_from_0_to_360(degree)
    # 計算結果を少数下4桁まででまるめる
    degree = (degree  * 10000).to_i.to_f / 10000
    return degree
  end

  # 度(degree)をラジアン(radian)に変換する。
  #
  # 【引数】
  #   1.度(degree:Float)
  # 【戻り値】
  #   1.ラジアン(radian:Float)
  #
  def degree_to_radian(degree)
    radian = degree / COEFFICIENT_OF_RADIAN_TO_DEGREE
    return radian
  end

  # 与えられた度(temporary_degree)を"0 <= n <360"内に収まった
  # 度(degree)に変換する。
  #
  # 【引数】
  #   1.度(temporary_degree:Float)
  # 【戻り値】
  #   1."0 <= n <360"内に収まった度(degree:Float)
  #
  def put_between_from_0_to_360(temporary_degree)
    while temporary_degree >= MAX_OF_DEGREE
      temporary_degree -= MAX_OF_DEGREE
    end
    while temporary_degree < MIN_OF_DEGREE
      temporary_degree += MAX_OF_DEGREE
    end
    degree = temporary_degree
    return degree
  end

  # 引数が度であるSINを算出する
  #
  # 【引数】
  #   1.度(degree:Float)
  # 【戻り値】
  #   1.SIN値(result:Float)
  # 【備考】
  #   戻り値は少数下4桁まででまるめたもの
  #
  def degree_sin(degree)
    result \
      = (Math.sin(self.degree_to_radian(degree)) * 10000).to_i.to_f / 10000
    return result
  end

  # 引数が度であるCOSを算出する
  #
  # 【引数】
  #   1.度(degree:Float)
  # 【戻り値】
  #   1.COS値(result:Float)
  # 【備考】
  #   戻り値は少数下4桁まででまるめたもの
  #
  def degree_cos(degree)
    result \
      = (Math.cos(self.degree_to_radian(degree)) * 10000).to_i.to_f / 10000
    return result
  end

  # 被除数（Floatクラス）を除数（Floatクラス）で割った時の
  # 剰余（Floatクラス）を求める
  #
  # 【引数】
  #   1.被除数(dividend:Float)
  #   2.除数(divisor:Float)
  # 【戻り値】
  #   1.剰余(surplus:Float)
  #
  def fmod(dividend, divisor)
    surplus = dividend.remainder(divisor)
    return surplus
  end

end
